Public Class frmOperators

#Region "Private Attributes"
    Private mCurrOperatorEdited As PolyMon.Operators.PMOperator = Nothing
    Private mCurrOperatorIsNew As Boolean = False
#End Region

#Region "Public Interface"
    Public Sub New()
        InitializeComponent()
        InitForm()
        SetEditState(False)
    End Sub
#End Region

#Region "Event Handlers"
    Private Sub tscOperators_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles tscOperators.SelectedIndexChanged
        ClearOperatorEditorFields()
        mCurrOperatorIsNew = False
        mCurrOperatorEdited = CType(Me.tscOperators.SelectedItem, PolyMon.Operators.PMOperator)
        With mCurrOperatorEdited
            Me.chkEnabled.Checked = .IsEnabled
            Me.txtName.Text = .Name
            Me.txtEmailAddress.Text = .EmailAddress
            Me.cboStartHour.Text = .OfflineTime.StartTime.Split(CChar(":"))(0)
            Me.cboStartMinute.Text = .OfflineTime.StartTime.Split(CChar(":"))(1)
            Me.cboEndHour.Text = .OfflineTime.EndTime.Split(CChar(":"))(0)
            Me.cboEndMinute.Text = .OfflineTime.EndTime.Split(CChar(":"))(1)
			Me.chkIncludeMessageBody.Checked = .IncludeMessageBody
			Me.cboQueuedNotify.SelectedIndex = .QueuedNotify
			Me.chkSummaryNotify.Checked = .SummaryNotify
			Me.chkSummaryOK.Checked = .SummaryNotifyOK
			Me.chkSummaryWarn.Checked = .SummaryNotifyWarn
			Me.chkSummaryFail.Checked = .SummaryNotifyFail
			Me.cboSummaryNotifyHour.Text = .SummaryNotifyTime.Split(CChar(":"))(0)
			Me.cboSummaryNotifyMinute.Text = .SummaryNotifyTime.Split(CChar(":"))(1)
		End With
        SetEditState(True)
    End Sub
    Private Sub tsbNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbNew.Click
        ClearOperatorEditorFields()
        mCurrOperatorIsNew = True
        mCurrOperatorEdited = New PolyMon.Operators.PMOperator()
        SetEditState(True)
    End Sub
    Private Sub tsbSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbSave.Click
        If mCurrOperatorEdited Is Nothing Then Exit Sub
        Try
            With mCurrOperatorEdited
                .IsEnabled = Me.chkEnabled.Checked
                .Name = Me.txtName.Text.Trim()
                .EmailAddress = Me.txtEmailAddress.Text.Trim()
                .OfflineTime.StartTime = Me.cboStartHour.Text & ":" & Me.cboStartMinute.Text
				.OfflineTime.EndTime = Me.cboEndHour.Text & ":" & Me.cboEndMinute.Text
                .IncludeMessageBody = Me.chkIncludeMessageBody.Checked
				.QueuedNotify = CType(Me.cboQueuedNotify.SelectedIndex, PolyMon.Operators.PMOperator.QueuedNotifyFlags)
				.SummaryNotify = Me.chkSummaryNotify.Checked
				.SummaryNotifyOK = Me.chkSummaryOK.Checked
				.SummaryNotifyWarn = Me.chkSummaryWarn.Checked
				.SummaryNotifyFail = Me.chkSummaryFail.Checked
				.SummaryNotifyTime = Me.cboSummaryNotifyHour.Text & ":" & Me.cboSummaryNotifyMinute.Text
                .Save()
            End With
            mCurrOperatorIsNew = False
            LoadOperators(mCurrOperatorEdited.OperatorID)
        Catch ex As Exception
            MsgBox("Error occurred while saving Operator:" & vbCrLf & ex.Message, MsgBoxStyle.Exclamation, "Error...")
        End Try
        SetEditState(True)
    End Sub
    Private Sub tsbDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbDelete.Click
        If Me.mCurrOperatorIsNew Then
            'Since this has not been saved yet, simply act like a cancel
            LoadOperators()
            SetEditState(False)
        Else
            If mCurrOperatorEdited Is Nothing Then
                ClearOperatorEditorFields()
                SetEditState(False)
            Else
                Dim ret As MsgBoxResult = MsgBox("You are about to delete an existing Operator." & vbCrLf & "Please note that this will delete the operator and ALL monitor associations." & vbCrLf & "Do you wish to continue?", MsgBoxStyle.Question Or MsgBoxStyle.YesNo, "Delete Operator?")
                If ret = MsgBoxResult.Yes Then
                    Try
                        Me.mCurrOperatorEdited.Delete()
                        LoadOperators()
                        SetEditState(False)
                    Catch ex As Exception
                        MsgBox("Error occurred deleting Operator." & vbCrLf & ex.Message, MsgBoxStyle.Exclamation, "Error...")
                    End Try
                Else
                    SetEditState(True)
                End If
            End If
        End If
    End Sub
    Private Sub tsbCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbCancel.Click
        If Me.mCurrOperatorIsNew Then
            LoadOperators()
            SetEditState(False)
        Else
            If mCurrOperatorEdited Is Nothing Then
                ClearOperatorEditorFields()
                SetEditState(False)
            Else
                LoadOperators(Me.mCurrOperatorEdited.OperatorID)
                SetEditState(True)
            End If

        End If
	End Sub
	Private Sub chkSummaryNotify_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkSummaryNotify.CheckedChanged
		Me.pnlSummaryNotify.Enabled = Me.chkSummaryNotify.Checked
	End Sub
#End Region

#Region "Private Methods"
    Private Sub InitForm()
        'Init Toolstrip Combo Box
        Dim cb As ComboBox = CType(Me.tscOperators.Control, ComboBox)
        cb.Items.Clear()
        cb.DisplayMember = "Name"
        cb.ValueMember = "OperatorID"

		'Init Offline Time Drop-Downs, Summary Notify Drop-Downs
        Dim i As Integer
        Dim cbHStart As ComboBox = CType(Me.cboStartHour, ComboBox)
        Dim cbMStart As ComboBox = CType(Me.cboStartMinute, ComboBox)
        Dim cbHEnd As ComboBox = CType(Me.cboEndHour, ComboBox)
		Dim cbMEnd As ComboBox = CType(Me.cboEndMinute, ComboBox)
		Dim cbSummaryH As ComboBox = CType(Me.cboSummaryNotifyHour, ComboBox)
		Dim cbSummaryM As ComboBox = CType(Me.cboSummaryNotifyMinute, ComboBox)

        cbHStart.Items.Clear()
        cbMStart.Items.Clear()
        cbHEnd.Items.Clear()
		cbMEnd.Items.Clear()
		cbSummaryH.Items.Clear()
		cbSummaryM.Items.Clear()
        For i = 0 To 23
            cbHStart.Items.Add(Format(i, "00"))
			cbHEnd.Items.Add(Format(i, "00"))
			cbSummaryH.Items.Add(Format(i, "00"))
        Next
        For i = 0 To 59 Step 10
            cbMStart.Items.Add(Format(i, "00"))
			cbMEnd.Items.Add(Format(i, "00"))
			cbSummaryM.Items.Add(Format(i, "00"))
        Next

		'Init Queued Notify Drop-Down
		Me.cboQueuedNotify.Items.Clear()
		Me.cboQueuedNotify.Items.Add("Do not send Queued Alerts")
		Me.cboQueuedNotify.Items.Add("Send all Queued Alerts")
		Me.cboQueuedNotify.Items.Add("Send Alert Recap Only")
		LoadOperators()

		Me.chkSummaryNotify.Checked = False
		Me.chkSummaryOK.Checked = False
		Me.chkSummaryWarn.Checked = False
		Me.chkSummaryFail.Checked = False
		Me.pnlSummaryNotify.Enabled = False
		Me.cboSummaryNotifyHour.SelectedIndex = 8
		Me.cboSummaryNotifyMinute.SelectedIndex = 0
    End Sub
    Private Sub ClearOperatorEditorFields()
        Me.mCurrOperatorEdited = Nothing
        Me.mCurrOperatorIsNew = False

        Me.chkEnabled.Checked = False
        Me.txtName.Text = Nothing
        Me.txtEmailAddress.Text = Nothing
        Me.cboStartHour.SelectedIndex = -1
        Me.cboStartMinute.SelectedIndex = -1
        Me.cboEndHour.SelectedIndex = -1
        Me.cboEndMinute.SelectedIndex = -1
		Me.chkIncludeMessageBody.Checked = False
		Me.cboQueuedNotify.SelectedIndex = 0

		Me.chkSummaryNotify.Checked = False
		Me.chkSummaryOK.Checked = False
		Me.chkSummaryWarn.Checked = False
		Me.chkSummaryFail.Checked = False
		Me.pnlSummaryNotify.Enabled = False
		Me.cboSummaryNotifyHour.SelectedIndex = 8
		Me.cboSummaryNotifyMinute.SelectedIndex = 0
    End Sub
    Private Sub LoadOperators(Optional ByVal OperatorID As Integer = -1)
        'Reset info
        ClearOperatorEditorFields()

        'Load Operators
        Me.tscOperators.Items.Clear()
        Dim PMOperators As New PolyMon.Operators.PMOperators
        Dim PMOperator As PolyMon.Operators.PMOperator

        For Each PMOperator In PMOperators
            Me.tscOperators.Items.Add(PMOperator)
        Next

        'Preselect Monitor Type if applicable
        If OperatorID > 0 Then
            For Each PMOperator In Me.tscOperators.Items
                If PMOperator.OperatorID = OperatorID Then
                    tscOperators.SelectedItem = PMOperator
                    Exit For
                End If
            Next
        End If
    End Sub
    Private Sub SetEditState(ByVal AllowEdits As Boolean)
        Me.chkEnabled.Enabled = AllowEdits
        Me.txtName.Enabled = AllowEdits
        Me.txtEmailAddress.Enabled = AllowEdits
        Me.cboStartHour.Enabled = AllowEdits
        Me.cboStartMinute.Enabled = AllowEdits
        Me.cboEndHour.Enabled = AllowEdits
        Me.cboEndMinute.Enabled = AllowEdits
        Me.chkIncludeMessageBody.Enabled = AllowEdits

        Me.tsbSave.Enabled = AllowEdits
        Me.tsbCancel.Enabled = AllowEdits
		Me.tsbDelete.Enabled = AllowEdits

		Me.cboQueuedNotify.Enabled = AllowEdits

		Me.chkSummaryNotify.Enabled = AllowEdits
		Me.pnlSummaryNotify.Enabled = AllowEdits And Me.chkSummaryNotify.Checked
	End Sub
#End Region


End Class